//=============================================================================
//
//		Copyright (C) 1995-1997 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscDelegateFlags.cc
//
//		Flags indicating which selectors a delegate responds to.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscDelegateFlags.cc,v 1.8 97/07/01 10:34:37 sunshine Exp $
// $Log:		MiscDelegateFlags.cc,v $
// Revision 1.8	 97/07/01  10:34:37	 sunshine
// v128: Fixed bug: Text color and highlighted text color delegate messages
// were reversed.  One was sent when the other should have been & vice versa.
// 
// Revision 1.7	 97/05/04  18:31:05	 zarnuk
// v126: fixed bug in objcToSel().
// fixed compiler warning.
// 
// Revision 1.6	 96/12/30  16:21:31	 zarnuk
// Added editing call backs.
//-----------------------------------------------------------------------------
#ifdef __GNUC__
# pragma implementation
#endif
#include "MiscDelegateFlags.h"
extern "C" {
#include <string.h>
}

static inline unsigned int BYTE_NUM( int x )	{ return (x / 8); }
static inline unsigned int BIT_NUM( int x )		{ return (x % 8); }
static inline unsigned char BIT_MASK( int x )	{ return (1 << BIT_NUM(x)); }

#define PSEL(X) &@selector(X)

// *** MUST MATCH ENUM IN .h FILE ***
static SEL const* const SELECTORS[ MiscDelegateFlags::MAX_DEL_ENUM ] = 
	{
	PSEL(tableScroll:border:slotDraggedFrom:to:),
	PSEL(tableScroll:border:slotSortReversed:),
	PSEL(tableScroll:border:slotResized:),
	PSEL(tableScroll:changeFont:to:),
	PSEL(tableScroll:fontChangedFrom:to:),
	PSEL(tableScroll:backgroundColorChangedTo:),
	PSEL(tableScroll:highlightBackgroundColorChangedTo:),
	PSEL(tableScroll:textColorChangedTo:),
	PSEL(tableScroll:highlightTextColorChangedTo:),
	PSEL(tableScroll:getISearchCol:),
	PSEL(tableScrollBuffCount:),
	PSEL(tableScroll:border:slotPrototype:),
	PSEL(tableScroll:border:slotTitle:),
	PSEL(tableScroll:cellAt: :),
	PSEL(tableScroll:reviveCell:at: :),
	PSEL(tableScroll:retireCell:at: :),
	PSEL(tableScroll:tagAt: :),
	PSEL(tableScroll:intValueAt: :),
	PSEL(tableScroll:floatValueAt: :),
	PSEL(tableScroll:doubleValueAt: :),
	PSEL(tableScroll:stringValueAt: :),
	PSEL(tableScroll:titleAt: :),
	PSEL(tableScroll:stateAt: :),
	PSEL(tableScrollRegisterServicesTypes:),
	PSEL(tableScroll:validRequestorForSendType:andReturnType:),
	PSEL(tableScroll:canWritePboardType:),
	PSEL(tableScroll:writePboard:type:toStream:),
	PSEL(tableScroll:writeSelectionToPasteboard:types:),
	PSEL(tableScroll:readSelectionFromPasteboard:),
	PSEL(tableScroll:allowDragOperationAt: :),
	PSEL(tableScroll:preparePasteboard:forDragOperationAt: :),
	PSEL(tableScroll:imageForDragOperationAt: :),
	PSEL(tableScroll:draggingSourceOperationMaskForLocal:),
	PSEL(tableScrollIgnoreModifierKeysWhileDragging:),
	PSEL(tableScrollWillPrint:),
	PSEL(tableScroll:didPrint:),
	PSEL(tableScroll:willPrintPageHeader:info:),
	PSEL(tableScroll:willPrintPageFooter:info:),
	PSEL(tableScroll:canEdit:at: :),
	PSEL(tableScroll:edit:at: :),
	PSEL(tableScroll:setStringValue:at: :),
	PSEL(tableScroll:abortEditAt: :),
	PSEL(tableScroll:willEditAt: :),
	PSEL(tableScroll:didEdit:at: :),
	PSEL(textDidEnd:endChar:),
	PSEL(textDidChange:),
	PSEL(textDidGetKeys:isEmpty:),
	PSEL(textWillChange:),
	PSEL(textWillEnd:),
	};

#undef PSEL

//-----------------------------------------------------------------------------
// selToObjc
//-----------------------------------------------------------------------------
SEL MiscDelegateFlags::selToObjc( Selector s )
	{
	return *SELECTORS[s];
	}


//-----------------------------------------------------------------------------
// objcToSel
//-----------------------------------------------------------------------------
MiscDelegateFlags::Selector MiscDelegateFlags::objcToSel( SEL s )
	{
	for (unsigned int i = 0; i < MAX_DEL_ENUM; i++)
		if (s == *SELECTORS[i])
			return (Selector)i;
	return BAD_DEL_ENUM;
	}


//-----------------------------------------------------------------------------
// setDelegate
//-----------------------------------------------------------------------------
void MiscDelegateFlags::setDelegate( id d )
	{
	if (d == 0)
		memset( set, 0, SET_SIZE );
	else
		{
		for (unsigned int i = 0; i < MAX_DEL_ENUM; i++)
			{
			unsigned char& byte = set[ BYTE_NUM(i) ];
			unsigned char const mask = BIT_MASK(i);
			if ([d respondsTo:(*SELECTORS[i])])
				byte |= mask;
			else
				byte &= ~mask;
			}
		}
	}


//-----------------------------------------------------------------------------
// respondsTo
//-----------------------------------------------------------------------------
bool MiscDelegateFlags::respondsTo( Selector s ) const
	{
	return ((set[ BYTE_NUM(s) ] & BIT_MASK(s)) != 0);
	}
